/////////////////////////////////////////
// FSNavigator SDK || GAUGE INTERFACE  //
// Use the file at your own risk.      //
/////////////////////////////////////////

/*
This INTERFACE allows normal Flight Simulator Gauges to access variables of
FSNavigator. The variables are made available as "Named Variables" of the
Flight Simulator gauge system. The description of the "Named Variables" can
be found in the Microsoft Flight Simulator SDK, Topic: Panels.

FSNavigator uses this Flight Simulator SDK functions:

void register_var_by_name(PVOID var, VAR_TYPE var_type, PSTRINGZ name);
void initialize_var_by_name(PMODULE_VAR module_var, PSTRINGZ name);

The access from a gauge can take place according to the following example.
The example simulates a using the "Fly Flightplan" function in FSNavigator.

PBOOL32 pFlyFlightplanOn = NULL;

void FSAPI update_routine()
{
	MODULE_VAR mvFlyFlightplanOn;

	if (pFlyFlightplanOn == NULL)
	{
		initialize_var_by_name(&mvFlyFlightplanOn, FSNAV_KEY_FLY_FLIGHTPLAN_ON);
		pFlyFlightplanOn = (PBOOL32)mvFlyFlightplanOn.var_ptr;
	}

	element_list_update( list );
}

void FSAPI flightplan_on_routine()
{
	if (pFlyFlightplanOn != NULL)
		*pFlyFlightplanOn = TRUE;
}

The Test is important: pFlyFlightplanOn != NULL
All variables of FSNavigator are only at disposal after FSNavigator has
been on time activated. Before initialize_var_by_name(module_var, name)
returns module_var.var_ptr NULL.

The variables of the FSNavigator are divided in 2 groups. The first group is
used for writing the values into FSNavigator. All their symbolic names begin
with FSNAV_KEY_. The second group is used for reading the values from
FSNavigator. All their symbolic names begin with FSNAV_VAR_.


Writting Variables
==================

Name:			FSNAV_KEY_FMC_MODE1_ON
Type:			TYPE_BOOL32
Description:	Activates Mode 1 inside FMC to setup Avionics.

Name:			FSNAV_KEY_FMC_MODE2_ON
Type:			TYPE_BOOL32
Description:	Activates Mode 2 inside FMC to setup Avionics.

Name:			FSNAV_KEY_FMC_MODE_TOGGLE
Type:			TYPE_BOOL32
Description:	Switches Mode 1 and 2 in FMC to setup Avionics.

Name:			FSNAV_KEY_FLY_TO_LATITUDE
Type:			TYPE_FLOAT64
Description:	Target Latitude for function "Fly To" (-90 ... +90)

Name:			FSNAV_KEY_FLY_TO_LONGITUDE
Type:			TYPE_FLOAT64
Description:	Target Longitude for function "Fly To" (-180 ... +180)

Name:			FSNAV_KEY_FLY_TO_ON
Type:			TYPE_BOOL32
Description:	Activates Function "Fly To". FSNAV_KEY_FLY_TO_LATITUDE
				and FSNAV_KEY_FLY_TO_LONGITUDE must be initialised before.

Name:			FSNAV_KEY_FLY_TO_OFF
Type:			TYPE_BOOL32
Description:	Deactivates function "Fly To".

Name:			FSNAV_KEY_FLY_TO_TOGGLE
Type:			TYPE_BOOL32
Description:	Activates or Deactivates function "Fly To". On activating
				FSNAV_KEY_FLY_TO_LATITUDE and FSNAV_KEY_FLY_TO_LONGITUDE
				must be initialised before.

Name:			FSNAV_KEY_FLY_FLIGHTPLAN_WP
Type:			TYPE_SINT32
Description:	Number of the first waypoint for the function "Fly Flightplan".
				With the default value -1 FSNavigator starts at the optimal waypoint.
				Normal uses only FSNavigator self this variable.

Name:			FSNAV_KEY_FLY_FLIGHTPLAN_ON
Type:			TYPE_BOOL32
Description:	Activates Function "Fly Flightplan".
				FSNAV_KEY_FLY_FLIGHTPLAN_WP can be, if needed initialised before.

Name:			FSNAV_KEY_FLY_FLIGHTPLAN_OFF
Type:			TYPE_BOOL32
Description:	Deactivates function "Fly Flightplan".

Name:			FSNAV_KEY_FLY_FLIGHTPLAN_TOGGLE
Type:			TYPE_BOOL32
Description:	Activates or Deactivates function "Fly Flightplan".
				FSNAV_KEY_FLY_FLIGHTPLAN_WP can be, if needed initialised before.

Name:			FSNAV_KEY_NEXT_WAYPOINT
Type:			TYPE_BOOL32
Description:	Next Waypoint for function "Fly Flightplan".

Name:			FSNAV_KEY_PREV_WAYPOINT
Type:			TYPE_BOOL32
Description:	Previous waypoint for function "Fly Flightplan".

Name:			FSNAV_KEY_FLY_HOLDING_ON
Type:			TYPE_BOOL32
Description:	Activated function "Fly Holding".

Name:			FSNAV_KEY_FLY_HOLDING_OFF
Type:			TYPE_BOOL32
Description:	Deactivated function "Fly Holding".

Name:			FSNAV_KEY_FLY_HOLDING_TOGGLE
Type:			TYPE_BOOL32
Description:	Activated or deactivates function "Fly Holding".


Reading Variables
=================

Name:			FSNAV_VAR_VERSION
Type:			TYPE_FLOAT64
Description:	The FSNavigator version number.

Name:			FSNAV_VAR_FMC_MODE2
Type:			TYPE_BOOL32
Description:	TRUE if Mode 2 to setup Avionics is active in FMC.

Name:			FSNAV_VAR_FLY_TO_POSSIBLE
Type:			TYPE_BOOL32
Description:	TRUE if an autopilot is available and the airplane is in air.

Name:			FSNAV_VAR_FLY_TO
Type:			TYPE_BOOL32
Description:	TRUE if function "Fly To" is active.

Name:			FSNAV_VAR_FLY_FP_POSSIBLE
Type:			TYPE_BOOL32
Description:	TRUE if an autopilot is available and the flight plan contains
				at least one waypoint.

Name:			FSNAV_VAR_FLY_FP
Type:			TYPE_BOOL32
Description:	TRUE if function "Fly Flightplan" is active.

Name:			FSNAV_VAR_FLY_HOLDING_POSSIBLE
Type:			TYPE_BOOL32
Description:	TRUE if an autopilot is available and Holding Pattern is defined.

Name:			FSNAV_VAR_FLY_HOLDING
Type:			TYPE_FLAGS32
Description:	Uses Flags FSNAV_FLY_HOLDING_ACTIVE and FSNAV_FLY_HOLDING_PRESSED.
				FSNAV_FLY_HOLDING_ACTIVE is set when function "Fly Holding" is
				active. FSNAV_FLY_HOLDING_PRESSED is set as long as the
				"Fly Holding" key is pressed in the FMS Window.

Name:			FSNAV_VAR_HSI_DATA
Type:			TYPE_VOID
Description:	module_var.var_ptr contains the address of FSNAV_HSI_DATA. This
				structure contains the data for the HSI.
*/

#ifdef __cplusplus
extern "C" {
#endif

// Symbolic names for variables to writing
#define FSNAV_KEY_FMC_MODE1_ON			"FSNAVIGATOR_FMC_MODE1_ON"
#define FSNAV_KEY_FMC_MODE2_ON			"FSNAVIGATOR_FMC_MODE2_ON"
#define FSNAV_KEY_FMC_MODE_TOGGLE		"FSNAVIGATOR_FMC_MODE_TOGGLE"

#define FSNAV_KEY_FLY_TO_LATITUDE		"FSNAVIGATOR_FLY_TO_LATITUDE"
#define FSNAV_KEY_FLY_TO_LONGITUDE		"FSNAVIGATOR_FLY_TO_LONGITUDE"
#define FSNAV_KEY_FLY_TO_ON				"FSNAVIGATOR_FLY_TO_ON"
#define FSNAV_KEY_FLY_TO_OFF			"FSNAVIGATOR_FLY_TO_OFF"
#define FSNAV_KEY_FLY_TO_TOGGLE			"FSNAVIGATOR_FLY_TO_TOGGLE"

#define FSNAV_KEY_FLY_FLIGHTPLAN_WP		"FSNAVIGATOR_FLY_FP_WP"
#define FSNAV_KEY_FLY_FLIGHTPLAN_ON		"FSNAVIGATOR_FLY_FP_ON"
#define FSNAV_KEY_FLY_FLIGHTPLAN_OFF	"FSNAVIGATOR_FLY_FP_OFF"
#define FSNAV_KEY_FLY_FLIGHTPLAN_TOGGLE	"FSNAVIGATOR_FLY_FP_TOGGLE"

#define FSNAV_KEY_NEXT_WAYPOINT			"FSNAVIGATOR_NEXT_WAYPOINT"
#define FSNAV_KEY_PREV_WAYPOINT			"FSNAVIGATOR_PREV_WAYPOINT"

#define FSNAV_KEY_FLY_HOLDING_ON		"FSNAVIGATOR_FLY_HOLDING_ON"
#define FSNAV_KEY_FLY_HOLDING_OFF		"FSNAVIGATOR_FLY_HOLDING_OFF"
#define FSNAV_KEY_FLY_HOLDING_TOGGLE	"FSNAVIGATOR_FLY_HOLDING_TOGGLE"

// Symbolic names for variables for reading
#define FSNAV_VAR_VERSION				"FSNAVIGATOR_VERSION"
#define FSNAV_VAR_FMC_MODE2				"FSNAVIGATOR_FMC_MODE2"
#define FSNAV_VAR_FLY_TO_POSSIBLE		"FSNAVIGATOR_FLY_TO_POSSIBLE"
#define FSNAV_VAR_FLY_TO				"FSNAVIGATOR_FLY_TO"
#define FSNAV_VAR_FLY_FP_POSSIBLE		"FSNAVIGATOR_FLY_FP_POSSIBLE"
#define FSNAV_VAR_FLY_FP				"FSNAVIGATOR_FLY_FP"
#define FSNAV_VAR_FLY_HOLDING_POSSIBLE	"FSNAVIGATOR_FLY_HOLDING_POSSIBLE"
#define FSNAV_VAR_FLY_HOLDING			"FSNAVIGATOR_FLY_HOLDING"
#define FSNAV_VAR_HSI_DATA				"FSNAVIGATOR_HSI_DATA"

// Status Bits for FSNAV_VAR_FLY_HOLDING
#define FSNAV_FLY_HOLDING_ACTIVE		0x00000001
#define FSNAV_FLY_HOLDING_PRESSED		0x00000002

#pragma pack(push, 1)
///////////////////////////////////////////////////

typedef struct FSNAV_HSI_DATA
{
	BOOL	m_bValid;
	double	m_fVerticalNeedle;				// -1.0 .. 1.0 corresponds to +5NM .. -5NM
	int		m_nTF_Flag;						// -1 - from. 0 - none, 1 - to
	double	m_fObiNeedle;					// 0 .. 360
	char	m_szStationName[6];
	char	m_szFrequency[7];
	double	m_fDistance;					// NM
}
FSNAV_HSI_DATA, *PFSNAV_HSI_DATA;

///////////////////////////////////////////////////
#pragma pack(pop)

#ifdef __cplusplus
}
#endif
